%%   Favago Transport Programming Interface - Hex_Lead_Hamiltonians
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%   Interface to create the Hamiltonian of one unit cell in a ribbon made of hexagonal lattice structure, including graphene and silicene.
%%
function handles = Hex_Lead_Hamiltonians()


handles.Graphene_Lead_Hamiltonians = @Graphene_Lead_Hamiltonians;
handles.Silicene_Lead_Hamiltonians = @Silicene_Lead_Hamiltonians;



%% Silicene_Lead_Hamiltonians
%   Creates Hamiltonians H_0 and H_1 of silicene ribbon with zigzag edge as well as the structure conatining the coordinates of the atomic sites.
function [H0, H1, coordinates] = Silicene_Lead_Hamiltonians(lead_param, M, Lead_End_Type, Atom_Type)
    % Hamiltonian created according to arXiv:1305.3684v1
    

    if isfield( lead_param, 'SOintrinsic' )
        SOintrinsic = -lead_param.SOintrinsic/(3*sqrt(3));
    else
        SOintrinsic = -0.3*vargamma/(3*sqrt(3));
    end
    if isfield( lead_param, 'deltaAB' )
        deltaAB = lead_param.deltaAB;
    else
        deltaAB = 0.2*vargamma;
    end
    if isfield( lead_param, 'SO_rashba_intrinsic' )
        SO_rashba_intrinsic = lead_param.SO_rashba_intrinsic*2/3;
    else
        SO_rashba_intrinsic = 0.07*vargamma;
    end
    
    
    
    if strcmp(Atom_Type, 'A')
        sgn = 1;
    else
        sgn = -1;
    end

    [H0,H1,coordinates] = Graphene_Lead_Hamiltonians(lead_param, M, Lead_End_Type, Atom_Type);
    coordinates.spinup = [ true( size(coordinates.x) ); false( size(coordinates.x) ) ];
    coordinates.x = [ coordinates.x; coordinates.x];
    coordinates.y = [ coordinates.y; coordinates.y];
    
    
    % intrinsic SO coupling
    HSOintrinsic_H0 = sparse( 1:2:M-1, M+2:2:2*M, -1i*SOintrinsic, size(H0,1), size(H0,2)) + sparse( 2:2:M, M+1:2:2*M-1, 1i*SOintrinsic, size(H0,1), size(H0,2)) + ...
                   sparse( 3:2:M, M+2:2:2*M-1, -1i*SOintrinsic, size(H0,1), size(H0,2)) + sparse( 2:2:M-1, M+3:2:2*M, 1i*SOintrinsic, size(H0,1), size(H0,2));
    HSOintrinsic_H0 = HSOintrinsic_H0 + HSOintrinsic_H0';
    
    HSOintrinsic_H1 = sparse( 1:2:M, 1:2:M, 1i*SOintrinsic, size(H1,1), size(H1,2)) + sparse( 2:2:M, 2:2:M, -1i*SOintrinsic, size(H1,1), size(H1,2)) + ...
                   sparse( M+1:2:2*M, M+1:2:2*M, -1i*SOintrinsic, size(H1,1), size(H1,2)) + sparse( M+2:2:2*M, M+2:2:2*M, 1i*SOintrinsic, size(H1,1), size(H1,2)) + ...
                   sparse( M+1:2:2*M, 2:2:M, 1i*SOintrinsic, size(H1,1), size(H1,2)) + sparse( M+2:2:2*M-1, 3:2:M, -1i*SOintrinsic, size(H1,1), size(H1,2)) + ...
                   sparse( M+3:2:2*M, 2:2:M-1, 1i*SOintrinsic, size(H1,1), size(H1,2)) + sparse( M+2:2:2*M, 1:2:M, -1i*SOintrinsic, size(H1,1), size(H1,2));
               
    % staggered A-B potential tuned by perpendicular electric field   
    deltaAB = deltaAB*sgn;
    H_DeltaAB = sparse( 1:2:M, 1:2:M, deltaAB, size(H0,1), size(H0,2)) + sparse( M+2:2:2*M, M+2:2:2*M, deltaAB, size(H0,1), size(H0,2)) + ...
            sparse( 2:2:M, 2:2:M, -deltaAB, size(H0,1), size(H0,2)) + sparse( M+1:2:2*M, M+1:2:2*M, -deltaAB, size(H0,1), size(H0,2));
              
    % intrinsic Rashba SO coupling
    % next-nearest vectors
    d1 = [coordinates.x(M+2) - coordinates.x(1); coordinates.y(M+2) - coordinates.y(1)]; d1 = d1/norm(d1);
    d2 = [coordinates.x(M+1) - coordinates.x(2); coordinates.y(M+1) - coordinates.y(2)]; d2 = d2/norm(d2);
    d3 = coordinates.a; d3 = d3/norm(d3);
    
    muvec = 0:M-1;
    muvec_firstrow = (-1).^(muvec) * sgn;
    muvec_secondrow = -muvec_firstrow;
    % matrix of spin updown
    HSO_rashba_intrinsic_H0 = ...
        sparse( M+2:2*M, 1:M-1, -1i*SO_rashba_intrinsic*(d1(2)+1i*d1(1))*muvec_secondrow(2:M), size(H0,1), size(H0,2)) + ...
        sparse( 1:M-1, M+2:2*M, -1i*SO_rashba_intrinsic*(-d1(2)+1i*(-d1(1)))*(muvec_firstrow(1:M-1)), size(H0,1), size(H0,2)) + ...
        sparse( M+1:2*M-1, 2:M, -1i*SO_rashba_intrinsic*(d2(2)+1i*d2(1))*muvec_secondrow(1:M-1), size(H0,1), size(H0,2)) + ...
        sparse( 2:M, M+1:2*M-1, -1i*SO_rashba_intrinsic*(-d2(2)+1i*(-d2(1)))*(muvec_firstrow(2:M)), size(H0,1), size(H0,2));
    HSO_rashba_intrinsic_H1 = ...
        sparse( 1:M, 1:M,         -1i*SO_rashba_intrinsic*(d3(2)+1i*d3(1))*muvec_firstrow, size(H1,1), size(H1,2)) + ...
        sparse( M+1:2*M, M+1:2*M, -1i*SO_rashba_intrinsic*(d3(2)+1i*d3(1))*muvec_secondrow, size(H1,1), size(H1,2)) + ...
        sparse( 2:M, M+1:2*M-1,         -1i*SO_rashba_intrinsic*(d1(2)+1i*d1(1))*muvec_firstrow(2:M), size(H1,1), size(H1,2)) + ...
        sparse( 1:M-1, M+2:2*M,         -1i*SO_rashba_intrinsic*(d2(2)+1i*d2(1))*muvec_firstrow(1:M-1), size(H1,1), size(H1,2));
    
    % matrix of spin downup    
    HSO_rashba_intrinsic_H0b = ...
        sparse( M+2:2*M, 1:M-1, -1i*SO_rashba_intrinsic*(d1(2)-1i*d1(1))*muvec_secondrow(2:M), size(H0,1), size(H0,2)) + ...
        sparse( 1:M-1, M+2:2*M, -1i*SO_rashba_intrinsic*(-d1(2)-1i*(-d1(1)))*muvec_firstrow(1:M-1), size(H0,1), size(H0,2)) + ...
        sparse( M+1:2*M-1, 2:M, -1i*SO_rashba_intrinsic*(d2(2)-1i*d2(1))*muvec_secondrow(1:M-1), size(H1,1), size(H0,2)) + ...
        sparse( 2:M, M+1:2*M-1, -1i*SO_rashba_intrinsic*(-d2(2)-1i*(-d2(1)))*muvec_firstrow(2:M), size(H1,1), size(H0,2));
    HSO_rashba_intrinsic_H1b = ...
        sparse( 1:M, 1:M,         -1i*SO_rashba_intrinsic*(d3(2)-1i*d3(1))*muvec_firstrow, size(H0,1), size(H1,2)) + ...
        sparse( M+1:2*M, M+1:2*M, -1i*SO_rashba_intrinsic*(d3(2)-1i*d3(1))*muvec_secondrow, size(H0,1), size(H1,2)); 
        sparse( 2:M, M+1:2*M-1,         -1i*SO_rashba_intrinsic*(d1(2)-1i*d1(1))*muvec_firstrow(2:M), size(H1,1), size(H1,2)) + ...
        sparse( 1:M-1, M+2:2*M,         -1i*SO_rashba_intrinsic*(d2(2)-1i*d2(1))*muvec_firstrow(1:M-1), size(H1,1), size(H1,2));
        
    
    % expand by spin up/down 
    H0 = [H0 + HSOintrinsic_H0 + H_DeltaAB, HSO_rashba_intrinsic_H0; ...
         HSO_rashba_intrinsic_H0b, H0 - HSOintrinsic_H0 + H_DeltaAB];
    H1 = [H1 + HSOintrinsic_H1, HSO_rashba_intrinsic_H1; ...
         HSO_rashba_intrinsic_H1b, H1 - HSOintrinsic_H1];
    
end

%% Graphene_Lead_Hamiltonians
% Creates Hamiltonians for graphene lead.
%   Creates Hamiltonians H_0 and H_1 of silicene ribbon with zigzag edge as well as the structure conatining the coordinates of the atomic sites.
function [H0, H1, coordinates] = Graphene_Lead_Hamiltonians(lead_param, M, Lead_End_Type, Atom_Type)
    
    if isfield( lead_param, 'epsilon' )
        epsilon = lead_param.epsilon;
    else
        epsilon = 0;
    end
    if isfield( lead_param, 'vargamma' )
        vargamma = lead_param.vargamma;
    else
        vargamma = 2.97;
    end

SegedfuggvenyekHandles = Segedfuggvenyek;

    if strcmp(Lead_End_Type, 'A')
        if strcmp(SegedfuggvenyekHandles.paritas(M),'ps') && strcmp(Atom_Type, 'B')
            tipus = 2;
        elseif strcmp(SegedfuggvenyekHandles.paritas(M),'ps') && strcmp(Atom_Type, 'A')
            tipus = 1;
        elseif strcmp(SegedfuggvenyekHandles.paritas(M),'pt') && strcmp(Atom_Type, 'A')
            tipus = 4;
        else
            tipus = 3;
        end
        
        [Hq,Hp,Hqp,h1,coordinates] = Armchair_End(epsilon,vargamma,M,tipus);
    elseif strcmp(Lead_End_Type, 'C')
        %if hanyadik_lead == 1; tipus = 3; 
        %elseif  hanyadik_lead == 2; tipus = 3;
        %end
        tipus = 3;
        [Hq,Hp,Hqp,h1,coordinates] = Cikk_cakk_End(epsilon,vargamma,M,tipus);
    else
        warning('Unrecognized Lead type, creating lead with zigzag edge');
        tipus = 3;
        [Hq,Hp,Hqp,h1,coordinates] = Armchair_End(epsilon,vargamma,M,tipus);
    end
    
    H0 = [Hp, Hqp'; Hqp, Hq];
    H1 = [zeros(size(Hp,1), size(Hp,2)+size(Hq,2)); h1', zeros(size(Hq)) ];
    
end

% Decimalatlan Hamilton_opok
% Armchair ended ribbon
function [Hq,Hp,Hqp,h1,coordinates] = Armchair_End(epsilon,vargamma,M,tipus)
	ny = M;

	coordinates = struct('x',zeros(2*ny,1),'y',zeros(2*ny,1), 'a', zeros(2,1) );
	deltax1 = 2;
	deltax2 = 1;
	deltay  = sqrt(3)/2;

    if tipus == 1 %osszeszukulo
        Hq  = eye(M)*epsilon + sparse(2:2:M-1,3:2:M,-vargamma,M,M) + sparse(2:2:M-1,3:2:M,-vargamma,M,M)';
        Hp  = eye(ny)*epsilon + sparse(1:2:ny,2:2:ny,-vargamma,ny,ny) + sparse(1:2:ny,2:2:ny,-vargamma,ny,ny)';
        Hqp = sparse(1:2:M-1,1:2:ny-1,-vargamma,M,ny) + sparse(2:2:M,2:2:ny,-vargamma,M,ny);
        h1  = sparse(1:ny,1:M,-vargamma,ny,M);
		
		coordinates.x(1:2:ny) = ((1:2:ny)-1)*(deltax1+deltax2)/2; %in units rCC
		coordinates.x(2:2:ny) = ((2:2:ny)-2)*(deltax1+deltax2)/2 + deltax2; %in units rCC
		coordinates.x(ny+(1:2:ny)) = ((1:2:ny)-1)*(deltax1+deltax2)/2 - deltax2/2; %in units rCC
		coordinates.x(ny+(2:2:ny)) = ((2:2:ny)-2)*(deltax1+deltax2)/2 + deltax1 - deltax2/2; %in units rCC        
		coordinates.y = [zeros(ny,1); ones(ny,1)*deltay]; %in units rCC
        coordinates.a = [0; 2*deltay];% 1D lattice constant in uits of rCC

    elseif tipus == 2  % kiszelesedo        
        Hq  = eye(M)*epsilon + sparse(1:2:M,2:2:M,-vargamma,M,M) + sparse(1:2:M,2:2:M,-vargamma,M,M)';
        Hp  = eye(ny)*epsilon + sparse(2:2:ny-2,3:2:ny,-vargamma,ny,ny) + sparse(2:2:ny-2,3:2:ny,-vargamma,ny,ny)';
        Hqp = sparse(2:2:M,2:2:ny,-vargamma,M,ny) + sparse(1:2:M-1,1:2:ny-1,-vargamma,M,ny); 
        h1  = sparse(1:ny,1:M,-vargamma,ny,M);    


		coordinates.x(1:2:ny) = ((1:2:ny)-1)*(deltax1+deltax2)/2; %in units rCC
		coordinates.x(2:2:ny) = ((2:2:ny)-2)*(deltax1+deltax2)/2 + deltax1; %in units rCC
		coordinates.x(ny+(1:2:ny)) = ((1:2:ny)-1)*(deltax1+deltax2)/2 + deltax2/2; %in units rCC 
        coordinates.x(ny+(2:2:ny)) = ((2:2:ny)-2)*(deltax1+deltax2)/2 + deltax2 + deltax2/2; %in units rCC  
        coordinates.y = [zeros(ny,1); ones(ny,1)*deltay]; %in units rCC
        coordinates.a = [0; 2*deltay];% 1D lattice constant in uits of rCC

    elseif tipus == 3 %felpup
        Hq  = eye(M)*epsilon + sparse(1:2:M-2,2:2:M,-vargamma,M,M) + sparse(1:2:M-2,2:2:M,-vargamma,M,M)';
        Hp  = eye(ny)*epsilon + sparse(2:2:ny,3:2:ny,-vargamma,ny,ny) + sparse(2:2:ny,3:2:ny,-vargamma,ny,ny)';
        Hqp = sparse(1:ny,1:M,-vargamma,ny,M);
        h1  = sparse(1:ny,1:M,-vargamma,ny,M);
        
        
  		coordinates.x(1:2:ny) = ((1:2:ny)-1)*(deltax1+deltax2)/2; %in units rCC
		coordinates.x(2:2:ny) = ((2:2:ny)-2)*(deltax1+deltax2)/2 + deltax1; %in units rCC
		coordinates.x(ny+(1:2:ny)) = ((1:2:ny)-1)*(deltax1+deltax2)/2 + deltax2/2; %in units rCC 
        coordinates.x(ny+(2:2:ny)) = ((2:2:ny)-2)*(deltax1+deltax2)/2 + deltax2 + deltax2/2; %in units rCC  
        coordinates.y = [zeros(ny,1); ones(ny,1)*deltay]; %in units rCC
        coordinates.a = [0; 2*deltay];% 1D lattice constant in uits of rCC

    elseif tipus == 4 %alulpup
        Hq  = eye(M)*epsilon + sparse(2:2:M,3:2:M,-vargamma,M,M) + sparse(2:2:M,3:2:M,-vargamma,M,M)';
        Hp  = eye(ny)*epsilon + sparse(1:2:ny-1,2:2:ny,-vargamma,ny,ny) + sparse(1:2:ny-2,2:2:ny,-vargamma,ny,ny)';
        Hqp = sparse(1:ny,1:M,-vargamma,ny,M);
        h1  = sparse(1:ny,1:M,-vargamma,ny,M);
        
        
   		coordinates.x(1:2:ny) = ((1:2:ny)-1)*(deltax1+deltax2)/2; %in units rCC
		coordinates.x(2:2:ny) = ((2:2:ny)-2)*(deltax1+deltax2)/2 + deltax2; %in units rCC
		coordinates.x(ny+(1:2:ny)) = ((1:2:ny)-1)*(deltax1+deltax2)/2 - deltax2/2; %in units rCC
		coordinates.x(ny+(2:2:ny)) = ((2:2:ny)-2)*(deltax1+deltax2)/2 + deltax1 - deltax2/2; %in units rCC 
        coordinates.y = [zeros(ny,1); ones(ny,1)*deltay]; %in units rCC
        coordinates.a = [0; 2*deltay];% 1D lattice constant in uits of rCC

    end
    
    %H0 = [Hq, Hqp; Hqp', Hp];
    %H1 = [zeros(size(Hq,1), size(Hq,2)+size(Hp,2)); h1, zeros(size(Hp)) ];
    
end

% zigzag ended ribbon
function [Hq,Hp,Hqp,h1,coordinates] = Cikk_cakk_End(epsilon,vargamma,M,tipus)

coordinates = struct('x',[],'y',[], 'a', zeros(2,1) );

Hp = eye(M)*epsilon;
if tipus == 1 %osseszukulo
    Meff = M + 2*(M-1);
    ny   = M-1;
    Hq   = sparse(1:M-1,M+1:M+ny,-vargamma,Meff,Meff) + sparse(2:M,M+1:M+ny,-vargamma,Meff,Meff) + sparse(M+1:Meff-ny,M+1+ny:Meff,-vargamma,Meff,Meff);
    Hq   = eye(Meff)*epsilon + Hq + Hq';
    Hqp  = sparse(Meff-ny+1:Meff,2:M,-vargamma,Meff,M) + sparse(Meff-ny+1:Meff,1:M-1,-vargamma,Meff,M);
    h1   = sparse(1:M,1:M,-vargamma,M,Meff);
elseif tipus == 2 %kitagulo
    Meff = M + 2*(M+1);
    ny   = M+1; 
    Hq   = sparse(1:M,M+2:M+ny,-vargamma,Meff,Meff) + sparse(1:M,M+1:M+ny-1,-vargamma,Meff,Meff) + sparse(M+1:Meff-ny,M+1+ny:Meff,-vargamma,Meff,Meff);
    Hq   = eye(Meff)*epsilon + Hq + Hq';
    Hqp  = sparse(Meff-ny+1:Meff-1,1:M,-vargamma,Meff,M) + sparse(Meff-ny+2:Meff,1:M,-vargamma,Meff,M);
    h1   = sparse(1:M,1:M,-vargamma,M,Meff);
elseif tipus == 3 %felul pup
    Meff = 3*M;
    ny   = M; 
    Hq   = sparse(1:M,M+1:M+ny,-vargamma,Meff,Meff) + sparse(2:M,M+1:M+ny-1,-vargamma,Meff,Meff) + sparse(M+1:Meff-ny,M+1+ny:Meff,-vargamma,Meff,Meff);
    Hq   = eye(Meff)*epsilon + Hq + Hq';
    Hqp  = sparse(Meff-ny+1:Meff-1,2:M,-vargamma,Meff,M) + sparse(Meff-ny+1:Meff,1:M,-vargamma,Meff,M);
    h1   = sparse(1:M,1:M,-vargamma,M,Meff); 

	coordinates.x = [ zeros(1,M); ones(1,M); ones(1,M)*3/2; ones(1,M)*5/2 ]; %in units rCC
	coordinates.y = [ (1:M)*sqrt(3); (1:M)*sqrt(3); (1:M)*sqrt(3)+sqrt(3)/2; (1:M)*sqrt(3)+sqrt(3)/2 ]; %in units rCC

elseif tipus == 4 %alul pup
    Meff = 3*M;
    ny   = M; 
    Hq   = sparse(1:M-1,M+2:M+ny,-vargamma,Meff,Meff) + sparse(1:M,M+1:M+ny,-vargamma,Meff,Meff) + sparse(M+1:Meff-ny,M+1+ny:Meff,-vargamma,Meff,Meff);
    Hq   = eye(Meff)*epsilon + Hq + Hq';
    Hqp  = sparse(Meff-ny+1:Meff,1:M,-vargamma,Meff,M) + sparse(Meff-ny+2:Meff,1:M-1,-vargamma,Meff,M);
    h1   = sparse(1:M,1:M,-vargamma,M,Meff);        
end

end

end
