%%    Favago Transport Programming Interface - Peierls
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%
%   Returns an object reasponsible for the Peierls and gauge transformations.
%%
function ret = Peierls(Opt, varargin)

    
parameter = [];
gauge     = []; % 'LandauX', 'LandauY' gauges or [] for load from input files
eta       = 0;  % eta_B = 2*pi/phi0*(rCC)^2*B;

inputparsing();

Ax              = [];
Ay              = [];
B_input         = [];
Vectorpotential = [];

inputparsing();

ret.PeierlsTransform      = @PeierlsTransform;
ret.PeierlsTransformLeads = @PeierlsTransformLeads;
ret.gaugeTransformation   = @gaugeTransformation;
ret.gaugeTransformationOnLead = @gaugeTransformationOnLead;
ret.setVectorPotential    = @setVectorPotential;


%% PeierlsTransformLeads
%   Algorithm to perform Peierls transformation in the leads.
function PeierlsTransformLeads( Surface_tmp, varargin )
    
    H0 = Surface_tmp.Read( 'H0' );
    Surface_tmp.Clear( 'H0' );
    H1 = Surface_tmp.Read( 'H1' );
    Surface_tmp.Clear( 'H1' );
    
    if Opt.BdG
        H0_size = size(H0,1)/2;
        mtx_pair_potential = H0( 1:H0_size, H0_size+1:2*H0_size);
        H0                 = H0( 1:H0_size, 1:H0_size);
        H1                 = H1( 1:H0_size, 1:H0_size);
        
        if norm(mtx_pair_potential,1) >= 1e-6
            error( 'The peierls transformation with nonzero B is forbidden in the superconducting phase. Set the pair potential to zero, or use gauge transformation instead.')
            mtx_pair_potential = mtx_pair_potential*0;
        end
    end
    
    
    if Opt.type_of_scanning == 1
        
        fazis_mtx_H0  = Surface_tmp.Read('fazis_mtx_H0');
        fazis_mtx_H1  = Surface_tmp.Read('fazis_mtx_H1');
        
        [sor_idx,oszlop_idx] = find(fazis_mtx_H0);
        Idx                  = length(sor_idx);
        idx                  = 1; 
        while idx <= Idx        
            H0(sor_idx(idx),oszlop_idx(idx)) = H0(sor_idx(idx),oszlop_idx(idx))*exp(1i*fazis_mtx_H0(sor_idx(idx),oszlop_idx(idx))*(parameter));        
            idx = idx + 1;
        end   
        
        [sor_idx,oszlop_idx] = find(fazis_mtx_H1);
        Idx                  = length(sor_idx);
        idx                  = 1; 
        while idx <= Idx        
            H1(sor_idx(idx),oszlop_idx(idx)) = H1(sor_idx(idx),oszlop_idx(idx))*exp(1i*fazis_mtx_H1(sor_idx(idx),oszlop_idx(idx))*(parameter));        
            idx = idx + 1;
        end   
       
    
 
    elseif Opt.type_of_scanning == 0       
        
        coordinates = Surface_tmp.Read( 'coordinates' );
    
        fazis_mtx_H0 = sparse([], [], 0, size(H0,1), size(H0,2) );
        fazis_mtx_H1 = sparse([], [], 0, size(H1,1), size(H1,2) );
    
    
        %Peierls in the unit cell
        [sor_idx,oszlop_idx] = find(H0);
        for idx = 1:length(sor_idx)
            kezdopont = [coordinates.x( oszlop_idx(idx) ); coordinates.y( oszlop_idx(idx) )];
            vegpont   = [coordinates.x( sor_idx(idx) ); coordinates.y( sor_idx(idx) )];
                if norm(vegpont-kezdopont) > 0
                    fazis                                         = Peierls_transzform(kezdopont,vegpont);
                   fazis_mtx_H0(sor_idx(idx),oszlop_idx(idx))    = fazis;
                   H0(sor_idx(idx),oszlop_idx(idx))              = exp(1i*fazis)*H0(sor_idx(idx),oszlop_idx(idx));
                end
        end
        Surface_tmp.Write( 'fazis_mtx_H0', fazis_mtx_H0);
    
    
        %Peierls in the coupling of unit cells
        [sor_idx,oszlop_idx] = find(H1);
        for idx = 1:length(sor_idx)
            kezdopont = [coordinates.x( oszlop_idx(idx) ); coordinates.y( oszlop_idx(idx) )] + coordinates.a;
            vegpont   = [coordinates.x( sor_idx(idx) ); coordinates.y( sor_idx(idx) )];
                if norm(vegpont-kezdopont) > 0
                   fazis                                         = Peierls_transzform(kezdopont,vegpont);
                   fazis_mtx_H1(sor_idx(idx),oszlop_idx(idx))    = fazis;
                   H1(sor_idx(idx),oszlop_idx(idx))              = exp(1i*fazis)*H1(sor_idx(idx),oszlop_idx(idx));
                end
        end    
        Surface_tmp.Write( 'fazis_mtx_H1', fazis_mtx_H1); 
    end
    
    if Opt.BdG
        H0 = [H0, mtx_pair_potential; mtx_pair_potential', -conj(H0)];
        H1 = [H1, zeros(size(H1)); zeros(size(H1')), -conj(H1)];
    end   
    
    
     Surface_tmp.Write( 'H0', H0 );
     H0 = [];
     Surface_tmp.Write( 'H1', H1 );
     Surface_tmp.Write( 'H1adj', H1' );
     H1 = [];   
    
    
end

%% PeierlsTransform
%   Algorithm to perform Peierls transformation in Hamiltonian stored in the Way2Hamiltonian object. 
function PeierlsTransform( Way2Hamiltonian )
    
    NameOfH = 'Hscatter';
    Hamiltoni = Way2Hamiltonian.Read( NameOfH );
    Way2Hamiltonian.Clear( NameOfH );

    if Opt.type_of_scanning == 1
        fazis_mtx  = Way2Hamiltonian.Read('fazis_mtx');        
        
        [sor_idx,oszlop_idx] = find(fazis_mtx);
        Idx                  = length(sor_idx);
        idx                  = 1; 
        while idx <= Idx        
            Hamiltoni(sor_idx(idx),oszlop_idx(idx)) = Hamiltoni(sor_idx(idx),oszlop_idx(idx))*exp(1i*fazis_mtx(sor_idx(idx),oszlop_idx(idx))*(parameter));        
            idx = idx + 1;
        end    
    
 
    elseif isempty(Opt.type_of_scanning)
        
        coordinates = Way2Hamiltonian.Read( 'coordinates' );    
        fazis_mtx = sparse([], [], 0, size(Hamiltoni,1), size(Hamiltoni,2) );
    
        %Peierls in the scattering region
        [sor_idx,oszlop_idx] = find(Hamiltoni);
        for idx = 1:length(sor_idx)
            kezdopont = [coordinates.x( oszlop_idx(idx) ); coordinates.y( oszlop_idx(idx) )];
            vegpont   = [coordinates.x( sor_idx(idx) ); coordinates.y( sor_idx(idx) )];
                if norm(vegpont-kezdopont) > 0
                    fazis                                         = Peierls_transzform(kezdopont,vegpont);
                    fazis_mtx(sor_idx(idx),oszlop_idx(idx))       = fazis;
                    Hamiltoni(sor_idx(idx),oszlop_idx(idx))       = exp(1i*fazis)*Hamiltoni(sor_idx(idx),oszlop_idx(idx));
                end
        end
        Way2Hamiltonian.Write( 'fazis_mtx', fazis_mtx);
        
    
        Ax      = [];
        Ay      = [];
        B_input = [];
    end
    
    Way2Hamiltonian.Write( NameOfH, Hamiltoni );
    Hamiltoni = [];
end


%% gaugeTransformation
%   Algorithm to perform gauge transformation on Greens function and/or on Hamiltonian.
    function ret = gaugeTransformation( mtx, coordinates, gauge_field )
       
        % for Landau_y = Landau_x + grad( gauge_field )
        %gauge_field = @(x,y)( eta*x.*y);
        
        mtx_size = size(mtx,1);
        if length( coordinates.x ) ~= mtx_size
            warning( 'The coordinates and matrix elements do not correspond to each other.' )
            return
        end
        
        % "u" and "v" like components must be transformed differently in BdG theory
        if isfield( coordinates, 'BdG_u' )
            fact = -(-1).^coordinates.BdG_u;
        else
            fact = ones( size(coordinates.x) );
        end
        
        if issparse( mtx )
            Umtx = sparse( 1:mtx_size, 1:mtx_size, exp(1i*fact.*gauge_field( coordinates.x, coordinates.y )), mtx_size, mtx_size);
        else
            Umtx = diag( exp(1i*fact.*gauge_field( coordinates.x, coordinates.y )) );
        end
        
        ret = Umtx * mtx * Umtx';        
        
    end


%% gaugeTransformationOnLead
%   Algorithm to perform gauge transformation in the Hmiltonians of a lead.
    function gaugeTransformationOnLead( Surface_tmp, gauge_field )   
        
        H0 = Surface_tmp.Read('H0');
        H1 = Surface_tmp.Read('H1');
        coordinates = Surface_tmp.Read( 'coordinates' );
        coordinates_tmp = struct( 'x', [], 'y', []);
        coordinates_tmp.x = [coordinates.x; coordinates.x+coordinates.a(1)];
        coordinates_tmp.y = [coordinates.y; coordinates.y+coordinates.a(2)];
        
        H0size = size(H0, 1);
        
        if isfield( coordinates, 'BdG_u' )
            coordinates_tmp.BdG_u = [coordinates.BdG_u; coordinates.BdG_u];
        end
        
        H_tmp = [H0,H1;H1',H0];%          
        H_tmp = gaugeTransformation( H_tmp, coordinates_tmp, gauge_field );
        
        
        H0 = H_tmp(1:H0size,1:H0size);
        H1 = H_tmp(1:H0size,H0size+1:end);   
        
        Surface_tmp.Write( 'H0', H0);
        Surface_tmp.Write( 'H1', H1);    
        Surface_tmp.Write( 'H1adj', H1');
        
        
    end
    
%% setVectorPotential
%   Use to set the handle for the vector potential.
function setVectorPotential( vectorpot )
	Vectorpotential = vectorpot;
end  

% This calculates the Peierls phase
function fazis = Peierls_transzform(kezdopont,vegpont)

fazis     = 0;    

if Opt.Linear_Regression_in_B
    A1        = Vectorpotential(kezdopont(1),kezdopont(2)); 
    A2        = Vectorpotential(vegpont(1),vegpont(2));  
    fazis     = 0.5*(A1(1)+A2(1))*(vegpont(1)-kezdopont(1)) + 0.5*(A1(2)+A2(2))*(vegpont(2)-kezdopont(2));
    return
end
    
if ~(kezdopont(1) == vegpont(1))
    dx        = (vegpont(1)-kezdopont(1))/100;
    int_array = kezdopont(1):dx:vegpont(1);
    Ind       = max(size(int_array));
    idx       = 1;
    A         = zeros(1,Ind);
    while idx <= Ind
        x      = int_array(idx);
        A_temp = Vectorpotential(x, y_temp(x,kezdopont,vegpont)); 
        A(idx) = A_temp(1);
        idx    = idx + 1;
    end
    fazis     = trapz(int_array,A);
end

if Opt.my_own_potential
    if ~(kezdopont(2) == vegpont(2) )
        dy        = (vegpont(2)-kezdopont(2))/100;
        int_array = kezdopont(2):dy:vegpont(2);
        Ind       = max(size(int_array));
        idx       = 1;
        A         = zeros(1,Ind);
        while idx <= Ind
            y      = int_array(idx);
            A_temp = Vectorpotential(x_temp(y,kezdopont,vegpont), y);
            A(idx) = A_temp(2);
            idx    = idx + 1;
        end
        fazis     = fazis + trapz(int_array,A);
    end
end
end

% create function for Vector potential
function HandleForA = CreateHandleForVectorpotential()    
        
    
    if ~isempty( gauge )  
        if strcmp( gauge, 'LandauX' )
            HandleForA = @(x,y)([-eta*y;0]);
            return
        elseif strcmp( gauge, 'LandauY' )
            HandleForA = @(x,y)([0;eta*x]);
            return
        end
    end
    
    LoadMagneticField();
    if Opt.my_own_potential
        HandleForA = @(x,y)my_vektorpot(x, y);
    else
        HandleForA = @(x,y)Vectorpot2D(x,y);
    end

end


function A = my_vektorpot(x,y)

[Ny,Nx]  = size(Ax);
I  = round(x);
J  = round(y);

if x <= I && x>1
    I = I-1;
end
if y <= J && y>1
    J = J-1;
end

lambdax = x-I;
lambday = y-J;

A = zeros(1,2);

if x<=1 && y<=1
    A(1) = Ax(1,1);
    A(2) = Ay(1,1);
elseif x<=1 && y>=Ny
    A(1) = Ax(Ny,1);
    A(2) = Ay(Ny,1);
elseif x<=1 && y>1 && y<Ny
    A(1) = (1-lambday)*Ax(J,1) + lambday*Ax(J+1,1);
    A(2) = (1-lambday)*Ay(J,1) + lambday*Ay(J+1,1);
elseif x>=Nx && y<=1
    A(1) = Ax(1,Nx);
    A(2) = Ay(1,Nx);
elseif x>=Nx && y>=Ny
    A(1) = Ax(Ny,Nx);
    A(2) = Ay(Ny,Nx);
elseif x>=Nx && y>1 && y<Ny
    A(1) = (1-lambday)*Ax(J,Nx) + lambday*Ax(J+1,Nx);
    A(2) = (1-lambday)*Ay(J,Nx) + lambday*Ay(J+1,Nx);
elseif x>1 && x<Nx && y<=1
    A(1) = (1-lambdax)*Ax(1,I) + lambdax*Ax(1,I+1);
    A(2) = (1-lambdax)*Ay(1,I) + lambdax*Ay(1,I+1);
elseif x>1 && x<Nx && y>=Ny
    A(1) = (1-lambdax)*Ax(Ny,I) + lambdax*Ax(Ny,I+1);
    A(2) = (1-lambdax)*Ay(Ny,I) + lambdax*Ay(Ny,I+1);
else
    A(1) = (1-lambday)*((1-lambdax)*Ax(J,I) + lambdax*Ax(J,I+1)) + lambday*((1-lambdax)*Ax(J+1,I) + lambdax*Ax(J+1,I+1));
    A(2) = (1-lambday)*((1-lambdax)*Ay(J,I) + lambdax*Ay(J,I+1)) + lambday*((1-lambdax)*Ay(J+1,I) + lambdax*Ay(J+1,I+1));
end
end

function Ax = Vectorpot2D(xv, yv)

y0 = 0;
dy = (yv-y0)/100;


if yv <= y0
   
    Ax       = 0;
else
    int_array  = y0:dy:yv;
    Idx        = size(int_array,2);
    int_values = zeros(1,Idx);
    idx        = 1;
    while idx <= Idx
        
        if strcmp(Opt.Lattice_Type, 'H')
            int_array_idx = [int_array(idx)*2/sqrt(3),x*2/3];
        else
            int_array_idx = [yv,xv];
        end
        
        int_values(idx) = B(int_array_idx(2),int_array_idx(1));
        idx             = idx + 1;
    end
    Ax       = -0.5*(trapz(int_array,int_values));
end
end
function indukcio = B(x,y)

[Ny, Nx] = size(B_input);

%B_input = [zeros(1,Nx+2);[zeros(Ny,1),B_input,zeros(Ny,1)];zeros(1,Nx+2)];
%x = x+1; y=y+1;
I = round(x);
J = round(y);

if x <= I && x>1
    I = I-1;
end
if y <= J && y>1
    J = J-1;
end

lambdax = x-I;
lambday = y-J;


if y <= 0 || y >= Ny || x <= 0 || x >= Nx
    indukcio = 0;
else
    indukcio = (1-lambday)*((1-lambdax)*B_input(J,I) + lambdax*B_input(J,I+1)) + lambday*((1-lambdax)*B_input(J+1,I) + lambdax*B_input(J+1,I+1));
end

end

function LoadMagneticField()
    if Opt.my_own_potential
        [Ax, Ay] = Load_A();
    else
        B_input = Load_B();
    end
end





% input parsing
function inputparsing()
    p_validating = inputParser;
    p_validating.addParamValue('parameter', 1);
    p_validating.addParamValue('Vectorpotential', []);
    p_validating.parse(varargin{:});
        
    parameter       = p_validating.Results.parameter;
    Vectorpotential = p_validating.Results.Vectorpotential;
    gauge           = [];
    eta             = 0;
    
    
    if isfield( Opt, 'eta' )
        eta   = Opt.eta;
    else
        eta = 0;
    end    
    
    
    if isempty( Vectorpotential )
        if isfield( Opt, 'gauge' )
            gauge     = Opt.gauge;
        else
            gauge = [];
        end
        Vectorpotential = CreateHandleForVectorpotential();
    end
        

end

end %Global end


% Loading Magnetic field
function [Ax, Ay] = Load_A()
    load Vector_Potential.mat Ax Ay;
    if sum(abs(size(Ax)-size(Ay))) ~= 0
        error('Sizes of Ax and Ay does not agree!!!')
    end
end

function B_input = Load_B()
    load B_input.mat B_input;
end
    
% Other fuctions for computing coordinates
function x = x_temp(y,kezdopont,vegpont)

x = vegpont(1) + (kezdopont(1)-vegpont(1))/(vegpont(2)-kezdopont(2))*(vegpont(2)-y);
end
function y = y_temp(x,kezdopont,vegpont)

y = vegpont(2) + (kezdopont(2)-vegpont(2))/(vegpont(1)-kezdopont(1))*(vegpont(1)-x);

end
