%%    Favago Transport Programming Interface - Decimation
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%
%   Returns an interface for calculations on a ribbon of finite size.
%%
function handles = Ribbon( varargin )

WS = ws_Ribbon;

InputParsing()
display('Creating ribbon object')

[ Opt, param ] = parseInput( WS.filenameIn );

G = []; % Greens function of the scattering region


createShape();
setParameters();
parseXML();
setFermiEnergy();
CreateHandles();

handles.GetWorkspace                = @GetWorkspace;
handles.ShiftLeads                  = @ShiftLeads;
handles.Transport                   = @Transport;
handles.CreateRibbon                = @CreateRibbon;
handles.CreateScatter               = @CreateScatter;
handles.setEnergy                   = @setEnergy;
handles.CalcFiniteGreensFunction    = @CalcFiniteGreensFunction;
handles.CustomDysonFunc             = @CustomDysonFunc;
handles.setHandlesForMagneticField  = @setHandlesForMagneticField;
handles.AddPNPotential              = @AddPNPotential;
handles.createPNpotential           = @createPNpotential;
handles.DecimationFunction          = @DecimationFunction;

%% Transport
% Calculate transport --- use for development pupose only in "contact/scattering center/contact" arrangement.
    function [Conductivity,aspect_ratio,Conductance,ny,DeltaC] = Transport(Energy)
        setEnergy( Energy )
        
        CalcFiniteGreensFunction();
        
        Dysonfunc = @()CustomDysonFunc();
        WS.FL_handles.DysonEq( 'CustomDyson', Dysonfunc );
        [S,ny] = WS.FL_handles.SmatrixCalc();
        
        try
            norma = norm(S*S'-eye(sum(ny)));
        catch
            norma = 1;
        end
        if norma >= 1e-3
            disp( ['error of the unitarity of S-matrix: ',num2str(norma)] )
        end
        
        if ny(1) ~= ny(2)
           disp( ['openchannels do not match: ',num2str(ny)] )
        end
        
        aspect_ratio = (WS.width*3/2)/((WS.height+3)*sqrt(3));
        
        conductance = WS.FL_handles.Conduktance();
        conductance = abs([conductance(1,:), conductance(2,:)]);
        DeltaC = std(conductance);
        
        C = mean(conductance);
        Conductivity = C/aspect_ratio;
        Conductance = C;
        
        disp( ['aspect ratio = ', num2str(aspect_ratio), ' conductance = ', num2str(C), ' conductivity = ', num2str(Conductivity)])        
        %2/pi
        
    end

%% CreateScatter
% Creates createH object for the the scattering region.
    function CreateScatter( )       
        CreateRibbon( 'justHamiltonians', true );
        createH = CreateHamiltonians(Opt, param);
        createH.CreateScatterH( 'Surface_tmp', WS.Surface_tmp );        
        WS.CreateH = createH;
        
    end

%% setEnergy
% Sets the energy for the calculations
    function setEnergy( Energy )
        WS.E = Energy;
        WS.FL_handles.setEnergy( WS.E );         
    end

%% ShiftLeads
% Shifts the insite energies in the leads by the given energy E
    function ShiftLeads( Energy )
        Surface = WS.FL_handles.Read( 'Surface' );
        
        for idx = 1:length(Surface)
            Surface{idx}.ShiftLead(Energy );
        end
        
    end




%% CustomDysonFunc
% Custom Dyson function for the "contact/scattering center/contact" arrangement.
    function Gret = CustomDysonFunc( varargin )
        
    p = inputParser;
    p.addParamValue('gfininv', inv(G));
    p.addParamValue('constant_channels', true);
    p.addParamValue('onlyGinverz', false );
    p.addParamValue('recalculateSurface', true );
    p.addParamValue('decimate', true );
    p.parse(varargin{:});
    gfininv     = p.Results.gfininv;
    constant_channels = p.Results.constant_channels;
    onlyGinverz        = p.Results.onlyGinverz;
    recalculateSurface = p.Results.recalculateSurface;  
    decimate           = p.Results.decimate; 
    
    if recalculateSurface
    
        if abs(WS.PNpotential) >= 1e-6
            pot = ribbon.createPNpotential();
        else
            pot = [];
        end
    
        % creating interfaces for the Leads
        if constant_channels
            shiftLeads = ones(length(param.Leads),1)*WS.E;
        else
            shiftLeads = ones(length(param.Leads),1)*0;
        end
    
        % creating objects for Leads
        if isempty( WS.Surface_interface )
            coordinates_shift = [-1, WS.height];
        else
            coordinates_shift = [-2, WS.height+1];
        end                    
        Surface = WS.FL_handles.LeadCalc('coordinates_shift', coordinates_shift, 'shiftLeads', shiftLeads, 'potential', pot);
    else
        Surface = WS.FL_handles.Read( 'Surface' );
    end
        
    %creating matrices for the Dyson Eq
    Ms = WS.FL_handles.Read('M');
    
    
    Gsurfinverz = cell(length(Ms),1);
    H1  = cell(length(Ms),1);
    H1adj = cell(length(Ms),1);
    for idx = 1:length(Ms)
            Gsurfinverz{idx} = Surface{idx}.Read('gsurfinv');%inv(gsurftemp);
            H1{idx}  = Surface{idx}.Read('H1');
            H1adj{idx}  = Surface{idx}.Read('H1adj');
    end
    
    
    H_interface = cell(length(Ms),1);
    H1_interface = cell(length( WS.Surface_interface ), 1);
    H1adj_interface = cell(length( WS.Surface_interface ), 1);
    for idx = 1:length( WS.Surface_interface )
            H_interface{idx} = WS.Surface_interface{idx}.Read('H0');  
            H1_interface{idx} = WS.Surface_interface{idx}.Read('H1');
            H1adj_interface{idx} = WS.Surface_interface{idx}.Read('H1adj');
    end  

    
    % The Dyson Eq
    GsurfinverzAll = zeros(sum(Ms), sum(Ms));
    H1_All = zeros(sum(Ms), sum(Ms));
    H1adj_All = zeros(sum(Ms), sum(Ms));
    if ~isempty( WS.Surface_interface )
        H_interface_All = zeros(sum(Ms), sum(Ms));
        H1_interface_All = zeros(sum(Ms), sum(Ms));
        H1adj_interface_All = zeros(sum(Ms), sum(Ms));
    end
    
    for idx = 1:length(Ms)
            GsurfinverzAll( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = Gsurfinverz{idx};
            Gsurfinverz{idx} = [];
            H1_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1{idx};
            H1{idx} = [];
            H1adj_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1adj{idx};
            H1adj{idx} = [];
            if ~isempty( WS.Surface_interface )
                H_interface_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H_interface{idx};
                H_interface{idx} = [];
                H1_interface_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1_interface{idx};
                H1_interface{idx} = [];
                H1adj_interface_All( sum(Ms(1:idx-1))+1:sum(Ms(1:idx)), sum(Ms(1:idx-1))+1:sum(Ms(1:idx)) ) = H1adj_interface{idx};
                H1adj_interface{idx} = [];                
            end
    end
    
    if ~isempty( WS.Surface_interface )
        Ginv_interface = WS.E*eye(size(H_interface_All)) - H_interface_All;
    end
        

    if ~isempty(WS.Surface_interface)
    	Ginverz = zeros(3*sum(Ms));
    	Ginverz(1:sum(Ms),1:sum(Ms)) = GsurfinverzAll;
	    Ginverz(1:sum(Ms),sum(Ms)+1:2*sum(Ms)) = -H1_All;
    	Ginverz(sum(Ms)+1:2*sum(Ms), 1:sum(Ms)) = -H1_All';
	    Ginverz(sum(Ms)+1:2*sum(Ms), sum(Ms)+1:2*sum(Ms)) = Ginv_interface;
    	Ginverz(sum(Ms)+1:2*sum(Ms),2*sum(Ms)+1:3*sum(Ms)) = -H1_interface_All;
	    Ginverz(2*sum(Ms)+1:3*sum(Ms), sum(Ms)+1:2*sum(Ms)) = -H1adj_interface_All;    
    	Ginverz(2*sum(Ms)+1:3*sum(Ms), 2*sum(Ms)+1:3*sum(Ms)) = gfininv;
    else
    	Ginverz = [ GsurfinverzAll, -H1_All; -H1_All', gfininv ];   
    end
    
    if decimate
        kulso_szabfokok = 1:sum(Ms);
        Ginverz = DecimationFunction( kulso_szabfokok, Ginverz);
    end
    
    if onlyGinverz
        Gret = [];
        return
    end
 
    Gret = inv( Ginverz );
        
    end

%% DecimationFunction
% Performs the decimation procedure on the inverse Green's Function
    function ret = DecimationFunction( kulso_szabfokok, ginv)
        
        CreateH = CreateHamiltonians(Opt, param);
        
        Hamiltoni = eye(size(ginv))*WS.E - ginv;
        clear ginv
    
        CreateH.Write('Hscatter', Hamiltoni);
        clear Hamiltoni;
    
        CreateH.Write('kulso_szabfokok', kulso_szabfokok);
        CreateH.Write('coordinates', []);
        
        Decimation_Procedure = Decimation(Opt); 
        Decimation_Procedure(WS.E, CreateH, 'Hscatter', 'kulso_szabfokok');
    
        Hamiltoni = CreateH.Read('Hscatter');
        CreateH.Clear('Hscatter');
    
        ret = eye(size(Hamiltoni))*WS.E - Hamiltoni;
        clear Hamiltoni;
        
        
    end




%% CalcFiniteGreensFunction
% Calculates the surface Greens fuction for a finite ribbon.
    function CalcFiniteGreensFunction( varargin )
    
    	p = inputParser;
        p.addParamValue('gauge_trans', false); % logical: true if want to perform gauge transformation on the Green's function and Hamiltonians       
        p.parse(varargin{:});
        gauge_trans     = p.Results.gauge_trans;
        
        CreateRibbon();  

        z1 = 0;
        z2 = WS.height;        
        
        WS.Surface_tmp.FiniteGreenFunction(z1,z2);
        
        G = WS.Surface_tmp.Read( 'gfin' );  
        
        % coordinates
        coordinates_tmp = WS.Surface_tmp.Read( 'coordinates' );
        
        coordinates_scatter.x = [coordinates_tmp.x+coordinates_tmp.a(1)*z1; coordinates_tmp.x+coordinates_tmp.a(1)*z2];
        coordinates_scatter.y = [coordinates_tmp.y+coordinates_tmp.a(2)*z1; coordinates_tmp.y+coordinates_tmp.a(2)*z2];
           
        
        % gauge transformation of the vector potential in the effective Hamiltonians
        if gauge_trans       
            try
            	% gauge transformation on Green's function
        		G = WS.PeierlsTransform_Scatter.gaugeTransformation( G, coordinates_scatter, WS.gauge_field );
        		% gauge transformation on the Hamiltonians
            	WS.PeierlsTransform_Scatter.gaugeTransformationOnLead( WS.Surface_tmp, WS.gauge_field );
            catch    
                error( 'Peierls tranformation in scattering region is not given. Unable to perform gauge transformation');    
            end
        end
        
    end


%% CreateRibbon
% Creates an interface for the ribbon shaped scattering region.
    function CreateRibbon( varargin )
        
        p = inputParser;
        p.addParamValue('justHamiltonians', false);         
        p.parse(varargin{:});
        justHamiltonians     = p.Results.justHamiltonians;
        
        
        display( 'Creating ribbon Hamiltonian.' )        
        
        WS.Surface_tmp.CreateHamiltonians( 'toSave', 0);
                
        % interface region is needed only when decimation is necessary in the ribbon hamiltonians to get invertable H1
        M = WS.Surface_tmp.Read('M');
        H0_size = size( WS.Surface_tmp.Read('H0'), 1);
        if M == H0_size
            WS.Surface_interface = [];
        end
        
        display('Creating interface Hamiltonians')
        coordinates_shift = [-1, WS.height ];
        for idx = 1:length(WS.Surface_interface)
            WS.Surface_interface{idx}.CreateHamiltonians( 'toSave', 0);
            WS.Surface_interface{idx}.ShiftCoordinates( coordinates_shift(idx) );
        end
        
        % apply magnetic field in scatter and interface
        ApplyMagneticField();    
        
        %WS.Surface_tmp.CalcSpektrum('ka_min', -pi, 'ka_max', pi);
        if justHamiltonians
            return
        end

        for idx = 1:length(WS.Surface_interface)
            DecimateInterface( idx );
        end
        
        display( 'Solving Hamiltonians in the scattering region' )        
        Decimation_Procedure_Lead = Decimation(Opt, 'ForcedDecimation', Opt.Decimation);
        Decimation_Procedure_Lead(WS.E, WS.Surface_tmp, 'Hamiltonian2Dec', 'kulso_szabfokok');  
        
        % Trukkos sajatertek
        display(['Eigenvalues for the scattering region'])
        WS.Surface_tmp.TrukkosSajatertekek(WS.E);
        % group velocity
        display(['Group velocity for the scattering region'])
        WS.Surface_tmp.Group_Velocity();
        
	
    end

%% ApplyMagneticField
% Applies the magnetic filed in the Hamiltonian of the scattering region.
    function ApplyMagneticField()        
        if ~isempty( WS.PeierlsTransform_Scatter )
            display('Applying magnetic field in ribbon Hamiltonians')
            WS.PeierlsTransform_Scatter.PeierlsTransformLeads( WS.Surface_tmp ); 
        end
        
        if ~isempty( WS.PeierlsTransform_Leads )            
            display('Applying magnetic field in the interface Hamiltonians')
            for idx = 1:length(WS.Surface_interface)
                if ~WS.Surface_interface{idx}.isSuperconducting()
                    WS.PeierlsTransform_Leads.PeierlsTransformLeads( WS.Surface_interface{idx} );  
                else
                    WS.PeierlsTransform_Leads.gaugeTransformationOnLead( WS.Surface_interface{idx}, WS.gauge_field );
                end
            end
        end   
    end


%% DecimateInterface
% Decimates the interface region (between the leads and the scattering region) for connecting the leads to scattering center (needed in hexagonal lattice).
    function DecimateInterface( idx )
        

        H00_scatter = WS.Surface_interface{idx}.Read('Hamiltonian2Dec');        
        H00size = size(H00_scatter,1);
        
        H0 = WS.Surface_tmp.Read('H0');
        
        % tranform the gauge in H0 if magnetic field is used
        if ~isempty( WS.PeierlsTransform_Scatter )
            %coordinates_tmp = WS.Surface_interface{idx}.Read('coordinates');%WS.Surface_tmp.Read( 'coordinates' );
            coordinates_tmp = WS.Surface_tmp.Read( 'coordinates' );
            if idx == 2
                coordinates_tmp.y = coordinates_tmp.y + (WS.height)*coordinates_tmp.a(2);
            end
                
            H0 = WS.PeierlsTransform_Scatter.gaugeTransformation( H0, coordinates_tmp, WS.gauge_field );
        end
        
        %H00_scatter( H00size/2+1:H00size, H00size/2+1:H00size ) = H0;
        H00_scatter( 1:H00size/2, 1:H00size/2 ) = H0;
        
        kulso_szabfokok = WS.Surface_interface{idx}.Read('kulso_szabfokok');
        coordinates = WS.Surface_interface{idx}.Read('coordinates');
        
        CreateH = CreateHamiltonians(Opt, param);
        %CreateH = WS.CreateH;        
        CreateH.Write('Hscatter', H00_scatter);
        clear H00_scatter;    
        CreateH.Write('kulso_szabfokok', kulso_szabfokok);
        CreateH.Write('coordinates', coordinates);

        Decimation_Procedure = Decimation(Opt); 
        Decimation_Procedure(WS.E, CreateH, 'Hscatter', 'kulso_szabfokok');
        
        H00_scatter = CreateH.Read('Hscatter');
        CreateH.Clear('Hscatter');      
        
        coordinates = WS.CreateH.Read('coordinates');
        WS.Surface_interface{idx}.Write('coordinates', coordinates);
        WS.Surface_interface{idx}.Write('Hamiltonian2Dec', H00_scatter); 
    end




%% setHandlesForMagneticField
% Creates the handles to apply magnetic filed in the Hamiltonians.
    function setHandlesForMagneticField( varargin )     
    
        p = inputParser;
        p.addParamValue('scatter', @(x,y)([0,0]) );
        p.addParamValue('lead', @(x,y)([0,0]) );
        p.addParamValue('gauge_field', @(x,y)(0) ); 
        
        p.parse(varargin{:});

       
        hscatter    = p.Results.scatter;
        hlead       = p.Results.lead;
        WS.gauge_field = p.Results.gauge_field;
        
        
        display('Adding handles of the magnetic field to the scattering region') 
        if isempty( WS.PeierlsTransform_Scatter )
        	WS.PeierlsTransform_Scatter = Peierls(Opt, 'Vectorpotential', hscatter);
        else
        	WS.PeierlsTransform_Scatter.setVectorPotential( hscatter );
        end
        
        if isempty( WS.PeierlsTransform_Leads ) 
        	WS.PeierlsTransform_Leads   = Peierls(Opt, 'Vectorpotential', hlead);
        else
        	WS.PeierlsTransform_Leads.setVectorPotential( hlead );
       	end
       	
    end


%% AddPNPotential
% Adds the p-n potential to the ribbon Hamiltonian
    function AddPNPotential( Surface_temp )
        if abs( WS.PNpotential ) <= 1e-6
            return
        end
        
        disp( 'Adding pn potential to the ribbon Hamiltonian' )
        H0 = Surface_temp.Read('H0');
        
        if 2*WS.width ~= size(H0,1)
            disp('WS.width ans size(H0) are in mismatch')
            return
        end
        pot = createPNpotential();
        Surface_temp.AddPotential( pot );
        
    end
        
        
        
%% createPNpotential
% Creates the p-n potential to be applied in the Hamiltonians
    function pot = createPNpotential()
        deltax_ps = 2;
        deltax_pt = 1; 
        deltax    = 3;
    
        pot = zeros(size(2*WS.width,1));
            
        xcoords1 = 1:WS.width;
        indexekpt = logical( mod(xcoords1,2)==1 );
        indexekps = ~indexekpt;            
        xcoords1( indexekpt ) = ( xcoords1( indexekpt )-1 )*deltax/2;
        xcoords1( indexekps ) = ( xcoords1( indexekps )-2 )*deltax/2 +deltax_pt;
            
        xcoords2 = 1:WS.width;   
        indexekpt = logical( mod(xcoords2,2)==1 );
        indexekps = ~indexekpt;     
        xcoords2( indexekpt ) = ( xcoords2( indexekpt )-1 )*deltax/2 - deltax_pt/2;
        xcoords2( indexekps ) = ( xcoords2( indexekps )-2 )*deltax/2 + deltax_ps - deltax_pt/2;
            
                                                              
        eta = WS.PNeta;    
        a = 2*WS.PNpotential/(max(xcoords1) - min(xcoords1))/eta;
        b = -WS.PNpotential/eta;
            
        pot( 1:WS.width ) = a*xcoords1 + b;
        pot( WS.width+1:2*WS.width ) = a*xcoords2 + b;
        
        indexek = logical(abs(pot) >= WS.PNpotential);
        pot(indexek) = sign(pot(indexek))*abs(WS.PNpotential);
            
            
	
    end
      

%% GetWorkspace()
% Return the handle class to the workspace of the given interface. Changes in fields of the workspace are not always safe, since reinitialization of function handles are not performed.
    function ret = GetWorkspace()
        ret = WS;        
    end

% setsFermiEnergy
% Sets the Fermi energy for the calculations on the sites.
    function setFermiEnergy()
        if ~isempty(WS.EF)
            param.scatter.epsilon = WS.EF;
            for idx = 1:length(param.Leads)
                param.Leads{idx}.epsilon = WS.EF;
            end
       end        
    end

% Creates handles for geometry operations
    function CreateHandles()  
        
        setHandlesForMagneticField();
        
        WS.CreateH = CreateHamiltonians(Opt, param);
        WS.FL_handles = Transport_Interface(WS.E, Opt, param, 'CreateH', WS.CreateH, 'PeierlsTransform', WS.PeierlsTransform_Leads );
        Opt = WS.FL_handles.Read( 'Opt' );
        WS.Surface_tmp = WS.FL_handles.SurfaceGreenFunctionCalculator([], 'createCore', 1);%, 'usingDualModes', 0);
        WS.Surface_interface = cell(length(param.Leads),1);
        for idx = 1:length(WS.Surface_interface)
            WS.Surface_interface{idx} = WS.FL_handles.SurfaceGreenFunctionCalculator(idx, 'createCore', 1);%, 'usingDualModes', 0);
        end
        
    end



    % Opens the XML file and fills up the parameter structures
    function parseXML()        
        WS.tree = xmlread(WS.filenameIn); 
        traverseNodes( WS.tree );  
        xmlwrite(WS.filenameOut,WS.tree);
    end


    % Parses over the attributes of the tree node
    function traverseNodes( theNode )
        % Recurse over node children.
        
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end
        

        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);
            
            if strcmp(char(theChild.getNodeName), 'parameters' )
                traverseNodes( theChild )
            elseif strcmp(char(theChild.getNodeName), 'computing_parameters' )
                setComputingParameters( theChild )
            elseif strcmp(char(theChild.getNodeName), 'Scatter_region' )
                setScatterNode( theChild ) 
            elseif strcmp(char(theChild.getNodeName), 'Lead_parameters' )
                setLeadParameters( theChild )
            end
            
        end
    end


% Sets computing parameters
    function setComputingParameters( theNode )
        
        setNode( theNode, 'Just_Create_Hamiltonians', 0);
        setNode( theNode,  'Decimate_only_Dyson', 1);
        setNode( theNode, 'magnetic_field', 1);
        setNode( theNode, 'scanning_parameter', 1);
        setNode( theNode, 'type_of_scanning', 1);
        setNode( theNode, 'lower_bound', 1);  
        setNode( theNode, 'step', 0);  
        setNode( theNode, 'number_of_steps', 1);  
        setNode( theNode, 'Linear_Regression_in_B', 1);  
        
        
    end


% Sets a value for a node in the xml tree under a specified node. If the node does not exit, this function creates it.
    function setNode( anchorNode, nodename, value, valueattribute)
        
        if ~exist('valueattribute', 'var')
            valueattribute = 'value';
        end
        
        Item = anchorNode.getElementsByTagName( nodename ).item(0);    
        
        if isempty( Item )
            Item = WS.tree.createElement( nodename );
            Item.setAttribute(valueattribute, num2str( value ));
            anchorNode.appendChild(Item);
        else
            Item.setAttribute(valueattribute, num2str( value ));
        end        
        
        
    end
        
    % Sets parameters for Leads
    function setLeadParameters( theNode )
        
        setNode( theNode, 'Hleads', 1);
        setNode( theNode, 'NofLeads', 2);
        
        %return
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end
        
        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);
            
            if strcmp(char(theChild.getNodeName), 'lead' )
                
                attributes = parseAttributes(theChild);
                
                if isfield(attributes, 'num')
                    idx = str2double(attributes.num);
                else
                    idx = idx + 1;
                    pointNode.theChild('num', num2str(idx));
                end
                
                if ~isempty(WS.EF)  
                    setNode( theChild, 'epsilon', WS.EF);   
                end
                
                if isfield( param.Leads{idx}, 'pair_potential' )
                	setNode( theChild, 'pair_potential', angle(param.Leads{idx}.pair_potential), 'phase');  
               	end
                
                
                
                
            end
            
        end
        
    end


    % Fills up data from one lead node
    function setScatterNode( theNode )
        
        setNode( theNode, 'Hscatter', 1);
        setNode( theNode, 'Hcoupling', 1);   
        if ~isempty(WS.EF)
            setNode( theNode, 'epsilon', WS.EF);   
        end       
        
        
        ShapeItem=theNode.getElementsByTagName('shape').item(0);  
        
        if isempty( ShapeItem )
            ShapeItem = WS.tree.createElement( 'shape' );
            theNode.appendChild(ShapeItem);
        end
        
        
        % creating shape node
        display('Creating shape node');
        setNode( ShapeItem, 'width', param.scatter.shape.width);
        setNode( ShapeItem, 'height', param.scatter.shape.height);
        setNode( ShapeItem, 'Lead_End_Type', param.scatter.shape.Lead_End_Type);
        setNode( ShapeItem, 'Atom_Type', param.scatter.shape.Atom_Type);
        
        return
        
 
        
    end


    % Parses the attributes of the XML node
    % ----- Local function PARSEATTRIBUTES -----
    function attributes = parseAttributes(theNode)
    % Create attributes structure.

    attributes = [];
    if theNode.hasAttributes
        theAttributes = theNode.getAttributes;
        numAttributes = theAttributes.getLength;
        attributes = struct();
    else
        return
    end

    for count = 1:numAttributes
        attrib = theAttributes.item(count-1);
        attributes.( char(attrib.getName) ) = char(attrib.getValue);
    end
    
    end

% creates graphene lead attach points
    function calculate_lead_attach_points()
        for idx = 1:length(param.Leads)
            param.Leads{idx}.M = WS.width;
        end
        %param.Leads.M           = zeros(2,1);
        %param.Leads.M(1) = WS.width;
        %param.Leads.M(2) = WS.width;
    end


% create the ribbon shape
    function createShape()
        calculate_lead_attach_points();
        
        param.scatter.shape.width = WS.width;
        param.scatter.shape.height = WS.height;
        param.scatter.shape.Lead_End_Type = 'A';
        param.scatter.shape.Atom_Type = 'A';       
    end


% Sets computing parameters
    function setParameters()
        Opt.Decimate_only_Dyson = 1;
        Opt.Hscatter = 0;
        Opt.Hcoupling = 0;
        Opt.Hleads = 0;
        Opt.saveWithMagneticFiled = 1;
        
        if isvector(WS.phi) && length(WS.phi) == length(param.Leads)
            for idx = 1:length(param.Leads)
                param.Leads{idx}.pair_potential = abs(param.Leads{idx}.pair_potential)*exp(1i*WS.phi(idx));
            end
        end
    end



% displays output messages on the screen
    function display( message )
       if WS.silent
           return
       else
           disp( message )
       end
        
    end

% Input parsing
    function InputParsing()
    
        p = inputParser;
        p.addParamValue('width', WS.width, @isscalar);
        p.addParamValue('height', WS.height, @isscalar);
        p.addParamValue('filenameIn', [pwd,  '/Basic_Input_zigzag_leads_orig.xml'], @ischar);
        p.addParamValue('filenameOut', [pwd,  '/Basic_Input_zigzag_leads.xml'], @ischar);
        p.addParamValue('E', WS.E, @isscalar);
        p.addParamValue('EF', WS.EF, @isscalar);
        p.addParamValue('phi', WS.phi);  
        p.addParamValue('silent', WS.silent);   
        p.addParamValue('PNpotential', WS.PNpotential, @isscalar);
        p.addParamValue('PNeta', WS.PNeta, @isscalar);        
        
        p.parse(varargin{:});

       
        WS.width     = p.Results.width;
        WS.height    = p.Results.height;
        WS.filenameIn   = p.Results.filenameIn;
        WS.filenameOut  = p.Results.filenameOut;
        WS.E            = p.Results.E;
        WS.EF           = p.Results.EF;
        WS.phi          = p.Results.phi;
        WS.silent       = p.Results.silent;
        WS.PNpotential  = p.Results.PNpotential;
        WS.PNeta  = p.Results.PNeta;        
        
        if mod(WS.width,2) == 1 && WS.width ~= 1
            display('correcting width to be even')
            WS.width = WS.width + 1;
        end
                

    end



end
